@@ -40,7 +40,6 @@ typedef const acb_struct * acb_srcptr;
 #define acb_realref(x) (&(x)->real)
 #define acb_imagref(x) (&(x)->imag)
 
-
 ACB_INLINE void
 acb_init(acb_t x)
 {
@@ -68,6 +67,13 @@ acb_get_imag(arb_t im, const acb_t z)
     arb_set(im, acb_imagref(z));
 }
 
+ACB_INLINE void
+acb_get_mid(acb_t res, const acb_t x)
+{
+    arb_get_mid_arb(acb_realref(res), acb_realref(x));
+    arb_get_mid_arb(acb_imagref(res), acb_imagref(x));
+}
+
 ACB_INLINE int
 acb_is_zero(const acb_t z)
 {
@@ -322,74 +328,9 @@ void acb_get_mag(mag_t z, const acb_t x);
 
 void acb_get_mag_lower(mag_t z, const acb_t x);
 
-ACB_INLINE void
-acb_get_abs_ubound_arf(arf_t u, const acb_t z, slong prec)
-{
-    if (arb_is_zero(acb_imagref(z)))
-    {
-        arb_get_abs_ubound_arf(u, acb_realref(z), prec);
-    }
-    else if (arb_is_zero(acb_realref(z)))
-    {
-        arb_get_abs_ubound_arf(u, acb_imagref(z), prec);
-    }
-    else
-    {
-        arf_t v;
-        arf_init(v);
-
-        arb_get_abs_ubound_arf(u, acb_realref(z), prec);
-        arb_get_abs_ubound_arf(v, acb_imagref(z), prec);
-
-        arf_mul(u, u, u, prec, ARF_RND_UP);
-        arf_mul(v, v, v, prec, ARF_RND_UP);
-        arf_add(u, u, v, prec, ARF_RND_UP);
-        arf_sqrt(u, u, prec, ARF_RND_UP);
-
-        arf_clear(v);
-    }
-}
-
-ACB_INLINE void
-acb_get_abs_lbound_arf(arf_t u, const acb_t z, slong prec)
-{
-    if (arb_is_zero(acb_imagref(z)))
-    {
-        arb_get_abs_lbound_arf(u, acb_realref(z), prec);
-    }
-    else if (arb_is_zero(acb_realref(z)))
-    {
-        arb_get_abs_lbound_arf(u, acb_imagref(z), prec);
-    }
-    else
-    {
-        arf_t v;
-        arf_init(v);
-
-        arb_get_abs_lbound_arf(u, acb_realref(z), prec);
-        arb_get_abs_lbound_arf(v, acb_imagref(z), prec);
-
-        arf_mul(u, u, u, prec, ARF_RND_DOWN);
-        arf_mul(v, v, v, prec, ARF_RND_DOWN);
-        arf_add(u, u, v, prec, ARF_RND_DOWN);
-        arf_sqrt(u, u, prec, ARF_RND_DOWN);
-
-        arf_clear(v);
-    }
-}
-
-ACB_INLINE void
-acb_get_rad_ubound_arf(arf_t u, const acb_t z, slong prec)
-{
-    /* fixme: this bound is very sloppy */
-
-    if (mag_cmp(arb_radref(acb_realref(z)), arb_radref(acb_imagref(z))) >= 0)
-        arf_set_mag(u, arb_radref(acb_realref(z)));
-    else
-        arf_set_mag(u, arb_radref(acb_imagref(z)));
-
-    arf_mul_2exp_si(u, u, 1);
-}
+void acb_get_abs_ubound_arf(arf_t u, const acb_t z, slong prec);
+void acb_get_abs_lbound_arf(arf_t u, const acb_t z, slong prec);
+void acb_get_rad_ubound_arf(arf_t u, const acb_t z, slong prec);
 
 ACB_INLINE void
 acb_union(acb_t res, const acb_t x, const acb_t y, slong prec)
@@ -404,6 +345,20 @@ void acb_sgn(acb_t res, const acb_t z, slong prec);
 
 void acb_csgn(arb_t res, const acb_t z);
 
+void acb_real_abs(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_sgn(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_heaviside(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_floor(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_ceil(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_real_max(acb_t res, const acb_t x, const acb_t y, int analytic, slong prec);
+void acb_real_min(acb_t res, const acb_t x, const acb_t y, int analytic, slong prec);
+void acb_real_sqrtpos(acb_t res, const acb_t z, int analytic, slong prec);
+
+void acb_sqrt_analytic(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_rsqrt_analytic(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_log_analytic(acb_t res, const acb_t z, int analytic, slong prec);
+void acb_pow_analytic(acb_t res, const acb_t z, const acb_t w, int analytic, slong prec);
+
 ACB_INLINE void
 acb_add(acb_t z, const acb_t x, const acb_t y, slong prec)
 {
@@ -630,6 +585,16 @@ acb_submul_arb(acb_t z, const acb_t x, const arb_t y, slong prec)
     arb_submul(acb_imagref(z), acb_imagref(x), y, prec);
 }
 
+void acb_dot_simple(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+void acb_dot_precise(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+void acb_dot(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+
+void acb_approx_dot(acb_t res, const acb_t initial, int subtract,
+    acb_srcptr x, slong xstep, acb_srcptr y, slong ystep, slong len, slong prec);
+
 void acb_inv(acb_t z, const acb_t x, slong prec);
 
 void acb_div(acb_t z, const acb_t x, const acb_t y, slong prec);
@@ -734,6 +699,24 @@ acb_coth(acb_t y, const acb_t x, slong prec)
     acb_mul_onei(y, y);
 }
 
+void acb_sech(acb_t r, const acb_t z, slong prec);
+void acb_csch(acb_t r, const acb_t z, slong prec);
+
+ACB_INLINE void
+acb_sec(acb_t y, const acb_t x, slong prec)
+{
+    acb_mul_onei(y, x);
+    acb_sech(y, y, prec);
+}
+
+ACB_INLINE void
+acb_csc(acb_t y, const acb_t x, slong prec)
+{
+    acb_mul_onei(y, x);
+    acb_csch(y, y, prec);
+    acb_mul_onei(y, y);
+}
+
 void acb_sin_pi(acb_t r, const acb_t z, slong prec);
 void acb_cos_pi(acb_t r, const acb_t z, slong prec);
 void acb_sin_cos_pi(acb_t s, acb_t c, const acb_t z, slong prec);
@@ -805,15 +788,6 @@ acb_sqr(acb_t res, const acb_t val, slong prec)
     acb_mul(res, val, val, prec);
 }
 
-/*
-TBD
-
-void acb_invroot_newton(acb_t r, const acb_t a, ulong m, const acb_t r0, slong startprec, slong prec);
-void acb_root_exp(acb_t r, const acb_t a, slong m, slong index, slong prec);
-void acb_root_newton(acb_t r, const acb_t a, slong m, slong index, slong prec);
-void acb_root(acb_t r, const acb_t a, slong m, slong index, slong prec);
-*/
-
 ACB_INLINE int
 acb_is_finite(const acb_t x)
 {
@@ -1038,6 +1012,7 @@ acb_rel_accuracy_bits(const acb_t x)
     return -acb_rel_error_bits(x);
 }
 
+slong acb_rel_one_accuracy_bits(const acb_t x);
 
 ACB_INLINE slong
 acb_bits(const acb_t x)
@@ -1074,23 +1049,7 @@ _acb_vec_bits(acb_srcptr vec, slong len)
     return _arb_vec_bits((arb_srcptr) vec, 2 * len);
 }
 
-ACB_INLINE void
-_acb_vec_set_powers(acb_ptr xs, const acb_t x, slong len, slong prec)
-{
-    slong i;
-
-    for (i = 0; i < len; i++)
-    {
-        if (i == 0)
-            acb_one(xs + i);
-        else if (i == 1)
-            acb_set_round(xs + i, x, prec);
-        else if (i % 2 == 0)
-            acb_mul(xs + i, xs + i / 2, xs + i / 2, prec);
-        else
-            acb_mul(xs + i, xs + i - 1, x, prec);
-    }
-}
+void _acb_vec_set_powers(acb_ptr xs, const acb_t x, slong len, slong prec);
 
 ACB_INLINE void
 _acb_vec_add_error_arf_vec(acb_ptr res, arf_srcptr err, slong len)
@@ -1142,7 +1101,7 @@ void _acb_vec_sort_pretty(acb_ptr vec, slong len);
 
 /* roots of unity */
 void acb_unit_root(acb_t res, ulong order, slong prec);
-void _acb_vec_unit_roots(acb_ptr z, slong len, slong prec);
+void _acb_vec_unit_roots(acb_ptr z, slong order, slong len, slong prec);
 
 ACB_INLINE slong
 acb_allocated_bytes(const acb_t x)
