@@ -22,10 +22,22 @@ acb_acos(acb_t res, const acb_t z, slong prec)
     {
         acb_t t;
         acb_init(t);
-        acb_asin(res, z, prec);
-        acb_const_pi(t, prec);
-        acb_mul_2exp_si(t, t, -1);
-        acb_sub(res, t, res, prec);
+        acb_one(t);
+        if (arb_is_zero(acb_imagref(z))
+                && arb_gt(acb_realref(z), acb_realref(t)))
+        {
+            /* pure imaginary on (1,inf) */
+            acb_asin(res, z, prec);
+            acb_neg(res, res);
+            arb_zero(acb_realref(res));
+        }
+        else
+        {
+            acb_asin(res, z, prec);
+            acb_const_pi(t, prec);
+            acb_mul_2exp_si(t, t, -1);
+            acb_sub(res, t, res, prec);
+        }
         acb_clear(t);
     }
 }
