@@ -13,7 +13,7 @@
 
 void
 acb_hypgeom_bessel_i_asymp_prefactors(acb_t A, acb_t B, acb_t C,
-    const acb_t nu, const acb_t z, slong prec)
+    const acb_t nu, const acb_t z, int scaled, slong prec)
 {
     acb_t t, u;
 
@@ -53,15 +53,26 @@ acb_hypgeom_bessel_i_asymp_prefactors(acb_t A, acb_t B, acb_t C,
         arb_union(acb_imagref(t), acb_imagref(t), acb_imagref(u), prec);
     }
 
-    acb_exp_invexp(B, A, z, prec);
-    acb_mul(A, A, t, prec);
+    if (scaled)
+    {
+        acb_neg(u, z);
+        acb_mul_2exp_si(u, u, 1);
+        acb_exp(u, u, prec);
+        acb_mul(A, t, u, prec);
+        acb_one(B);
+    }
+    else
+    {
+        acb_exp_invexp(B, A, z, prec);
+        acb_mul(A, A, t, prec);
+    }
 
     acb_clear(t);
     acb_clear(u);
 }
 
 void
-acb_hypgeom_bessel_i_asymp(acb_t res, const acb_t nu, const acb_t z, slong prec)
+acb_hypgeom_bessel_i_asymp(acb_t res, const acb_t nu, const acb_t z, int scaled, slong prec)
 {
     acb_t A1, A2, C, U1, U2, s, t, u;
     int is_real, is_imag;
@@ -79,7 +90,7 @@ acb_hypgeom_bessel_i_asymp(acb_t res, const acb_t nu, const acb_t z, slong prec)
     is_real = acb_is_real(nu) && acb_is_real(z)
         && (acb_is_int(nu) || arb_is_positive(acb_realref(z)));
 
-    if (!is_real && arb_is_zero(acb_realref(z)) && acb_is_int(nu))
+    if (!is_real && !scaled && arb_is_zero(acb_realref(z)) && acb_is_int(nu))
     {
         acb_mul_2exp_si(t, nu, -1);
 
@@ -89,7 +100,10 @@ acb_hypgeom_bessel_i_asymp(acb_t res, const acb_t nu, const acb_t z, slong prec)
             is_imag = 1;
     }
 
-    acb_hypgeom_bessel_i_asymp_prefactors(A1, A2, C, nu, z, prec);
+    if (scaled)
+        is_imag = 0;
+
+    acb_hypgeom_bessel_i_asymp_prefactors(A1, A2, C, nu, z, scaled, prec);
 
     /* todo: if Ap ~ 2^a and Am = 2^b and U1 ~ U2 ~ 1, change precision? */
 
@@ -134,7 +148,7 @@ acb_hypgeom_bessel_i_asymp(acb_t res, const acb_t nu, const acb_t z, slong prec)
 }
 
 void
-acb_hypgeom_bessel_i_0f1(acb_t res, const acb_t nu, const acb_t z, slong prec)
+acb_hypgeom_bessel_i_0f1(acb_t res, const acb_t nu, const acb_t z, int scaled, slong prec)
 {
     acb_struct b[2];
     acb_t w, c, t;
@@ -143,7 +157,7 @@ acb_hypgeom_bessel_i_0f1(acb_t res, const acb_t nu, const acb_t z, slong prec)
     {
         acb_init(t);
         acb_neg(t, nu);
-        acb_hypgeom_bessel_i_0f1(res, t, z, prec);
+        acb_hypgeom_bessel_i_0f1(res, t, z, scaled, prec);
         acb_clear(t);
         return;
     }
@@ -169,6 +183,13 @@ acb_hypgeom_bessel_i_0f1(acb_t res, const acb_t nu, const acb_t z, slong prec)
 
     acb_hypgeom_pfq_direct(t, NULL, 0, b, 2, w, -1, prec);
 
+    if (scaled)
+    {
+        acb_neg(w, z);
+        acb_exp(w, w, prec);
+        acb_mul(t, t, w, prec);
+    }
+
     acb_mul(res, t, c, prec);
 
     acb_clear(b + 0);
@@ -188,9 +209,26 @@ acb_hypgeom_bessel_i(acb_t res, const acb_t nu, const acb_t z, slong prec)
 
     if (mag_cmp_2exp_si(zmag, 4) < 0 ||
         (mag_cmp_2exp_si(zmag, 64) < 0 && 2 * mag_get_d(zmag) < prec))
-        acb_hypgeom_bessel_i_0f1(res, nu, z, prec);
+        acb_hypgeom_bessel_i_0f1(res, nu, z, 0, prec);
     else
-        acb_hypgeom_bessel_i_asymp(res, nu, z, prec);
+        acb_hypgeom_bessel_i_asymp(res, nu, z, 0, prec);
+
+    mag_clear(zmag);
+}
+
+void
+acb_hypgeom_bessel_i_scaled(acb_t res, const acb_t nu, const acb_t z, slong prec)
+{
+    mag_t zmag;
+
+    mag_init(zmag);
+    acb_get_mag(zmag, z);
+
+    if (mag_cmp_2exp_si(zmag, 4) < 0 ||
+        (mag_cmp_2exp_si(zmag, 64) < 0 && 2 * mag_get_d(zmag) < prec))
+        acb_hypgeom_bessel_i_0f1(res, nu, z, 1, prec);
+    else
+        acb_hypgeom_bessel_i_asymp(res, nu, z, 1, prec);
 
     mag_clear(zmag);
 }
