@@ -1,30 +1,16 @@
-/*=============================================================================
-
-    This file is part of ARB.
-
-    ARB is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    ARB is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with ARB; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
-
-=============================================================================*/
-/******************************************************************************
-
+/*
     Copyright (C) 2014 Fredrik Johansson
 
     2x2 mul code taken from MPFR 2.3.0
     (Copyright (C) 1991-2007 Free Software Foundation, Inc.)
 
-******************************************************************************/
+    This file is part of Arb.
+
+    Arb is free software: you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 2.1 of the License, or
+    (at your option) any later version.  See <http://www.gnu.org/licenses/>.
+*/
 
 #ifndef ARF_H
 #define ARF_H
@@ -35,11 +21,18 @@
 #define ARF_INLINE static __inline__
 #endif
 
+#include <stdio.h>
 #include <math.h>
-#include "flint.h"
+#include "flint/flint.h"
 #include "fmpr.h"
 #include "mag.h"
 
+#ifndef flint_abort
+#if __FLINT_RELEASE <= 20502
+#define flint_abort abort
+#endif
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -241,12 +234,7 @@ arf_init(arf_t x)
     ARF_XSIZE(x) = 0;
 }
 
-ARF_INLINE void
-arf_clear(arf_t x)
-{
-    fmpz_clear(ARF_EXPREF(x));
-    ARF_DEMOTE(x);
-}
+void arf_clear(arf_t x);
 
 ARF_INLINE void
 arf_zero(arf_t x)
@@ -361,6 +349,16 @@ int arf_cmp(const arf_t x, const arf_t y);
 
 int arf_cmpabs(const arf_t x, const arf_t y);
 
+int arf_cmpabs_ui(const arf_t x, ulong y);
+
+int arf_cmpabs_d(const arf_t x, double y);
+
+int arf_cmp_si(const arf_t x, slong y);
+
+int arf_cmp_ui(const arf_t x, ulong y);
+
+int arf_cmp_d(const arf_t x, double y);
+
 ARF_INLINE void
 arf_swap(arf_t y, arf_t x)
 {
@@ -441,10 +439,13 @@ arf_init_set_ui(arf_t x, ulong v)
     }
 }
 
+/* FLINT_ABS is unsafe for x = WORD_MIN */
+#define UI_ABS_SI(x) (((slong)(x) < 0) ? (-(ulong)(x)) : ((ulong)(x)))
+
 ARF_INLINE void
 arf_init_set_si(arf_t x, slong v)
 {
-    arf_init_set_ui(x, FLINT_ABS(v));
+    arf_init_set_ui(x, UI_ABS_SI(v));
     if (v < 0)
         ARF_NEG(x);
 }
@@ -473,19 +474,11 @@ arf_set_ui(arf_t x, ulong v)
 ARF_INLINE void
 arf_set_si(arf_t x, slong v)
 {
-    arf_set_ui(x, FLINT_ABS(v));
+    arf_set_ui(x, UI_ABS_SI(v));
     if (v < 0)
         ARF_NEG(x);
 }
 
-ARF_INLINE int
-arf_cmpabs_ui(const arf_t x, ulong y)
-{
-    arf_t t;
-    arf_init_set_ui(t, y);  /* no need to free */
-    return arf_cmpabs(x, t);
-}
-
 ARF_INLINE void
 arf_init_set_shallow(arf_t z, const arf_t x)
 {
@@ -557,10 +550,8 @@ arf_set_fmpz(arf_t y, const fmpz_t x)
 
 int _arf_set_round_ui(arf_t x, ulong v, int sgnbit, slong prec, arf_rnd_t rnd);
 
-/* TODO: document */
 int _arf_set_round_uiui(arf_t z, slong * fix, mp_limb_t hi, mp_limb_t lo, int sgnbit, slong prec, arf_rnd_t rnd);
 
-/* Assumes xn > 0, x[xn-1] != 0. */
 int
 _arf_set_round_mpn(arf_t y, slong * exp_shift, mp_srcptr x, mp_size_t xn,
     int sgnbit, slong prec, arf_rnd_t rnd);
@@ -574,7 +565,7 @@ arf_set_round_ui(arf_t x, ulong v, slong prec, arf_rnd_t rnd)
 ARF_INLINE int
 arf_set_round_si(arf_t x, slong v, slong prec, arf_rnd_t rnd)
 {
-    return _arf_set_round_ui(x, FLINT_ABS(v), v < 0, prec, rnd);
+    return _arf_set_round_ui(x, UI_ABS_SI(v), v < 0, prec, rnd);
 }
 
 ARF_INLINE int
@@ -675,39 +666,9 @@ arf_bot(fmpz_t e, const arf_t x)
         fmpz_sub_si(e, ARF_EXPREF(x), arf_bits(x));
 }
 
-ARF_INLINE int
-arf_is_int(const arf_t x)
-{
-    if (arf_is_special(x))
-        return arf_is_zero(x);
-    else
-    {
-        fmpz_t t;
-        int r;
-        fmpz_init(t);
-        arf_bot(t, x);
-        r = fmpz_sgn(t) >= 0;
-        fmpz_clear(t);
-        return r;
-    }
-}
+int arf_is_int(const arf_t x);
 
-ARF_INLINE int
-arf_is_int_2exp_si(const arf_t x, slong e)
-{
-    if (arf_is_special(x))
-        return arf_is_zero(x);
-    else
-    {
-        fmpz_t t;
-        int r;
-        fmpz_init(t);
-        arf_bot(t, x);
-        r = fmpz_cmp_si(t, e) >= 0;
-        fmpz_clear(t);
-        return r;
-    }
-}
+int arf_is_int_2exp_si(const arf_t x, slong e);
 
 int arf_cmp_2exp_si(const arf_t x, slong e);
 
@@ -783,35 +744,22 @@ arf_abs_bound_le_2exp_fmpz(fmpz_t b, const arf_t x)
 
 slong arf_abs_bound_lt_2exp_si(const arf_t x);
 
+void arf_frexp(arf_t man, fmpz_t exp, const arf_t x);
+
 void arf_get_fmpz_2exp(fmpz_t man, fmpz_t exp, const arf_t x);
 
-void arf_get_fmpz(fmpz_t z, const arf_t x, arf_rnd_t rnd);
+int _arf_get_integer_mpn(mp_ptr y, mp_srcptr x, mp_size_t xn, slong exp);
+
+int _arf_set_mpn_fixed(arf_t z, mp_srcptr xp, mp_size_t xn,
+        mp_size_t fixn, int negative, slong prec, arf_rnd_t rnd);
+
+int arf_get_fmpz(fmpz_t z, const arf_t x, arf_rnd_t rnd);
 
 slong arf_get_si(const arf_t x, arf_rnd_t rnd);
 
-ARF_INLINE int
-arf_get_fmpz_fixed_fmpz(fmpz_t y, const arf_t x, const fmpz_t e)
-{
-    int r;
-    fmpr_t t;
-    fmpr_init(t);
-    arf_get_fmpr(t, x);
-    r = fmpr_get_fmpz_fixed_fmpz(y, t, e);
-    fmpr_clear(t);
-    return r;
-}
+int arf_get_fmpz_fixed_fmpz(fmpz_t y, const arf_t x, const fmpz_t e);
 
-ARF_INLINE int
-arf_get_fmpz_fixed_si(fmpz_t y, const arf_t x, slong e)
-{
-    int r;
-    fmpr_t t;
-    fmpr_init(t);
-    arf_get_fmpr(t, x);
-    r = fmpr_get_fmpz_fixed_si(y, t, e);
-    fmpr_clear(t);
-    return r;
-}
+int arf_get_fmpz_fixed_si(fmpz_t y, const arf_t x, slong e);
 
 ARF_INLINE void
 arf_set_fmpz_2exp(arf_t x, const fmpz_t man, const fmpz_t exp)
@@ -827,9 +775,21 @@ void arf_ceil(arf_t z, const arf_t x);
 
 void arf_debug(const arf_t x);
 
-void arf_print(const arf_t x);
+void arf_fprint(FILE * file, const arf_t x);
 
-void arf_printd(const arf_t y, slong d);
+void arf_fprintd(FILE * file, const arf_t y, slong d);
+
+ARF_INLINE void
+arf_print(const arf_t x)
+{
+    arf_fprint(stdout, x);
+}
+
+ARF_INLINE void
+arf_printd(const arf_t y, slong d)
+{
+    arf_fprintd(stdout, y, d);
+}
 
 void arf_randtest(arf_t x, flint_rand_t state, slong bits, slong mag_bits);
 
@@ -907,7 +867,7 @@ void arf_randtest_special(arf_t x, flint_rand_t state, slong bits, slong mag_bit
 extern TLS_PREFIX mp_ptr __arf_mul_tmp;
 extern TLS_PREFIX slong __arf_mul_alloc;
 
-extern void _arf_mul_tmp_cleanup(void);
+ARB_DLL extern void _arf_mul_tmp_cleanup(void);
 
 #define ARF_MUL_TMP_DECL \
     mp_limb_t tmp_stack[ARF_MUL_STACK_ALLOC]; \
@@ -1003,7 +963,7 @@ arf_mul_fmpz(arf_ptr z, arf_srcptr x, const fmpz_t y, slong prec, arf_rnd_t rnd)
 extern TLS_PREFIX mp_ptr __arf_add_tmp;
 extern TLS_PREFIX slong __arf_add_alloc;
 
-extern void _arf_add_tmp_cleanup(void);
+ARB_DLL extern void _arf_add_tmp_cleanup(void);
 
 #define ARF_ADD_TMP_DECL \
     mp_limb_t tmp_stack[ARF_ADD_STACK_ALLOC]; \
@@ -1109,6 +1069,8 @@ arf_submul_fmpz(arf_ptr z, arf_srcptr x, const fmpz_t y, slong prec, arf_rnd_t r
         return arf_submul_mpz(z, x, COEFF_TO_PTR(*y), prec, rnd);
 }
 
+int arf_sosq(arf_t z, const arf_t x, const arf_t y, slong prec, arf_rnd_t rnd);
+
 int arf_div(arf_ptr z, arf_srcptr x, arf_srcptr y, slong prec, arf_rnd_t rnd);
 
 ARF_INLINE int
@@ -1258,7 +1220,7 @@ arf_mag_add_ulp(mag_t z, const mag_t x, const arf_t y, slong prec)
     if (ARF_IS_SPECIAL(y))
     {
         flint_printf("error: ulp error not defined for special value!\n");
-        abort();
+        flint_abort();
     }
     else if (MAG_IS_LAGOM(z) && MAG_IS_LAGOM(x) && ARF_IS_LAGOM(y))
     {
@@ -1280,7 +1242,7 @@ arf_mag_set_ulp(mag_t z, const arf_t y, slong prec)
     if (ARF_IS_SPECIAL(y))
     {
         flint_printf("error: ulp error not defined for special value!\n");
-        abort();
+        flint_abort();
     }
     else
     {
@@ -1314,6 +1276,25 @@ int arf_sum(arf_t s, arf_srcptr terms, slong len, slong prec, arf_rnd_t rnd);
 double arf_get_d(const arf_t x, arf_rnd_t rnd);
 void arf_set_d(arf_t x, double v);
 
+ARF_INLINE slong
+arf_allocated_bytes(const arf_t x)
+{
+    slong size = fmpz_allocated_bytes(ARF_EXPREF(x));
+
+    if (ARF_HAS_PTR(x))
+        size += ARF_PTR_ALLOC(x) * sizeof(mp_limb_t);
+
+    return size;
+}
+
+int arf_load_str(arf_t res, const char * data);
+
+char * arf_dump_str(const arf_t x);
+
+int arf_load_file(arf_t res, FILE *stream);
+
+int arf_dump_file(FILE* stream, const arf_t x);
+
 #ifdef __cplusplus
 }
 #endif
