@@ -24,6 +24,7 @@ mag_log1p(mag_t z, const mag_t x)
     }
     else
     {
+        double t;
         fmpz exp = MAG_EXP(x);
 
         if (!COEFF_IS_MPZ(exp))
@@ -32,50 +33,33 @@ mag_log1p(mag_t z, const mag_t x)
             if (exp < -10)
             {
                 mag_set(z, x);
-                return;
             }
             else if (exp < 1000)
             {
-                double t;
                 t = ldexp(MAG_MAN(x), exp - MAG_BITS);
                 t = (1.0 + t) * (1 + 1e-14);
                 t = mag_d_log_upper_bound(t);
                 mag_set_d(z, t);
-                return;
+            }
+            else
+            {
+                /* log(2^(exp-1) * (2*v)) = exp*log(2) + log(2*v), 1 <= 2v < 2 */
+                t = (MAG_MAN(x) + 1) * ldexp(1.0, 1 - MAG_BITS);
+                t = mag_d_log_upper_bound(t);
+                t += (exp - 1) * 0.69314718055994530942;
+                t *= (1 + 1e-13);
+                mag_set_d(z, t);
             }
         }
         else if (fmpz_sgn(MAG_EXPREF(x)) < 0)
         {
             /* Quick bound by x */
             mag_set(z, x);
-            return;
         }
-
-        /* Now we must have x >= 2^1000 */
-        /* Use log(2^(exp-1) * (2*v)) = exp*log(2) + log(2*v) */
+        else
         {
-            double t;
-            fmpz_t b;
-            mag_t u;
-
-            mag_init(u);
-            fmpz_init(b);
-
-            /* incrementing the mantissa gives an upper bound for x+1 */
-            t = ldexp(MAG_MAN(x) + 1, 1 - MAG_BITS);
-            t = mag_d_log_upper_bound(t);
-            mag_set_d(u, t);
-
-            /* log(2) < 744261118/2^30 */
-            _fmpz_add_fast(b, MAG_EXPREF(x), -1);
-            fmpz_mul_ui(b, b, 744261118);
-            mag_set_fmpz(z, b);
-            _fmpz_add_fast(MAG_EXPREF(z), MAG_EXPREF(z), -30);
-
-            mag_add(z, z, u);
-
-            mag_clear(u);
-            fmpz_clear(b);
+            mag_add_ui(z, x, 1);
+            mag_log(z, z);
         }
     }
 }
