@@ -42,7 +42,7 @@ int main()
         mag_get_fmpr(x, xb);
         mag_get_fmpr(y, yb);
 
-        fmpr_div(z, x, y, MAG_BITS + 10, FMPR_RND_DOWN);
+        fmpr_div(z, x, y, MAG_BITS + 10, FMPR_RND_UP);
         fmpr_mul_ui(z2, z, 1025, MAG_BITS, FMPR_RND_UP);
         fmpr_mul_2exp_si(z2, z2, -10);
 
